"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MarkdownConverter = void 0;
const tslib_1 = require("tslib");
const form_data_1 = tslib_1.__importDefault(require("form-data"));
const common_1 = require("../../common");
const converter_utils_1 = require("../utils/converter.utils");
const converter_1 = require("./converter");
const main_config_1 = require("../../main.config");
/**
 * Class representing a Markdown converter that extends the base Converter class.
 * This class is used to convert HTML with markdown content to PDF using Gotenberg service.
 *
 * @extends Converter
 */
class MarkdownConverter extends converter_1.Converter {
    /**
     * Creates an instance of MarkdownConverter.
     * Initializes the converter with the Markdown conversion route.
     */
    constructor() {
        super(main_config_1.ChromiumRoute.MARKDOWN);
    }
    /**
     * Converts HTML with markdown content to PDF.
     *
     * @param {Object} options - Conversion options.
     * @param {PathLikeOrReadStream} options.html - PathLike or ReadStream of the HTML content to be converted.
     * @param {PathLikeOrReadStream} options.markdown - PathLike or ReadStream of the Markdown content to be converted.
     * @param {PathLikeOrReadStream} [options.header] - PathLike or ReadStream of the header HTML content.
     * @param {PathLikeOrReadStream} [options.footer] - PathLike or ReadStream of the footer HTML content.
     * @param {PageProperties} [options.properties] - Page properties for the conversion.
     * @param {PdfFormat} [options.pdfFormat] - PDF format options.
     * @param {boolean} [options.pdfUA] - Indicates whether to generate PDF/UA compliant output.
     * @param {EmulatedMediaType} [options.emulatedMediaType] - Emulated media type for the conversion.
     * @param {string} [options.waitDelay] - Delay before the conversion process starts.
     * @param {string} [options.waitForExpression] - JavaScript expression to wait for before completing the conversion.
     * @param {string} [options.userAgent] - User agent string to use during the conversion.
     * @param {Record<string, string>} [options.extraHttpHeaders] - Additional HTTP headers for the conversion.
     * @param {number []} [options.failOnHttpStatusCodes] - Whether to fail on HTTP status code.
     * @param {boolean} [options.failOnConsoleExceptions] - Whether to fail on console exceptions during conversion.
     * @param {boolean} [options.skipNetworkIdleEvent] - Whether to skip network idle event.
     * @param {Metadata} options.metadata - Metadata to be written.
     * @returns {Promise<Buffer>} A Promise resolving to the converted PDF content as a Buffer.
     */
    convert(_a) {
        return tslib_1.__awaiter(this, arguments, void 0, function* ({ html, markdown, header, footer, properties, pdfFormat, pdfUA, emulatedMediaType, waitDelay, waitForExpression, userAgent, extraHttpHeaders, failOnHttpStatusCodes, failOnConsoleExceptions, skipNetworkIdleEvent, metadata }) {
            const data = new form_data_1.default();
            yield common_1.GotenbergUtils.addFile(data, html, 'index.html');
            yield common_1.GotenbergUtils.addFile(data, markdown, 'file.md');
            yield converter_utils_1.ConverterUtils.customize(data, {
                header,
                footer,
                properties,
                pdfFormat,
                pdfUA,
                emulatedMediaType,
                waitDelay,
                waitForExpression,
                userAgent,
                extraHttpHeaders,
                failOnHttpStatusCodes,
                failOnConsoleExceptions,
                skipNetworkIdleEvent,
                metadata
            });
            return common_1.GotenbergUtils.fetch(this.endpoint, data);
        });
    }
}
exports.MarkdownConverter = MarkdownConverter;
//# sourceMappingURL=markdown.converter.js.map