"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UrlConverter = void 0;
const tslib_1 = require("tslib");
const url_1 = require("url");
const form_data_1 = tslib_1.__importDefault(require("form-data"));
const common_1 = require("../../common");
const converter_utils_1 = require("../utils/converter.utils");
const converter_1 = require("./converter");
const main_config_1 = require("../../main.config");
/**
 * Class representing a URL converter that extends the base Converter class.
 * This class is used to convert content from a URL to PDF using Gotenberg service.
 *
 * @extends Converter
 */
class UrlConverter extends converter_1.Converter {
    /**
     * Creates an instance of UrlConverter.
     * Initializes the converter with the URL conversion route.
     */
    constructor() {
        super(main_config_1.ChromiumRoute.URL);
    }
    /**
     * Converts content from a URL to PDF.
     *
     * @param {Object} options - Conversion options.
     * @param {string} options.url - The URL of the content to be converted to PDF.
     * @param {PathLikeOrReadStream} [options.header] - PathLike or ReadStream of the header HTML content.
     * @param {PathLikeOrReadStream} [options.footer] - PathLike or ReadStream of the footer HTML content.
     * @param {PageProperties} [options.properties] - Page properties for the conversion.
     * @param {PdfFormat} [options.pdfFormat] - PDF format options.
     * @param {boolean} [options.pdfUA] - Indicates whether to generate PDF/UA compliant output.
     * @param {EmulatedMediaType} [options.emulatedMediaType] - Emulated media type for the conversion.
     * @param {string} [options.waitDelay] - Delay before the conversion process starts.
     * @param {string} [options.waitForExpression] - JavaScript expression to wait for before completing the conversion.
     * @param {string} [options.userAgent] - User agent string to use during the conversion.
     * @param {Record<string, string>} [options.extraHttpHeaders] - Additional HTTP headers for the conversion.
     * @param {number[]} [options.failOnHttpStatusCodes] - Whether to fail on HTTP status code.
     * @param {boolean} [options.failOnConsoleExceptions] - Whether to fail on console exceptions during conversion.
     * @param {boolean} [options.skipNetworkIdleEvent] - Whether to skip network idle event.
     * @param {Metadata} options.metadata - Metadata to be written.
     * @returns {Promise<Buffer>} A Promise resolving to the converted PDF content as a Buffer.
     */
    convert(_a) {
        return tslib_1.__awaiter(this, arguments, void 0, function* ({ url, header, footer, properties, pdfFormat, pdfUA, emulatedMediaType, waitDelay, waitForExpression, userAgent, extraHttpHeaders, failOnHttpStatusCodes, failOnConsoleExceptions, skipNetworkIdleEvent, metadata }) {
            const _url = new url_1.URL(url);
            const data = new form_data_1.default();
            data.append('url', _url.href);
            yield converter_utils_1.ConverterUtils.customize(data, {
                header,
                footer,
                properties,
                pdfFormat,
                pdfUA,
                emulatedMediaType,
                waitDelay,
                waitForExpression,
                userAgent,
                extraHttpHeaders,
                failOnHttpStatusCodes,
                failOnConsoleExceptions,
                skipNetworkIdleEvent,
                metadata
            });
            return common_1.GotenbergUtils.fetch(this.endpoint, data);
        });
    }
}
exports.UrlConverter = UrlConverter;
//# sourceMappingURL=url.converter.js.map