"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScreenshotUtils = void 0;
const tslib_1 = require("tslib");
const common_1 = require("../../common");
/**
 * Utility class for handling common tasks related to screenshot.
 */
class ScreenshotUtils {
    /**
     * Adds page properties to the FormData object based on the provided imageProperties.
     *
     * @param {FormData} data - The FormData object to which page properties will be added.
     * @param {ImageProperties} imageProperties - The page properties to be added to the FormData.
     */
    static addImageProperties(data, imageProperties) {
        data.append('format', imageProperties.format);
        if (imageProperties.quality) {
            common_1.GotenbergUtils.assert(imageProperties.format === 'jpeg', 'Compression quality is exclusively supported for JPEG format.');
            common_1.GotenbergUtils.assert(imageProperties.quality >= 0 && imageProperties.quality <= 100, 'Invalid compression quality. Please provide a value between 0 and 100.');
            data.append('quality', imageProperties.quality);
        }
        if (imageProperties.omitBackground) {
            data.append('omitBackground', String(imageProperties.omitBackground));
        }
    }
    /**
     * Customizes the FormData object based on the provided screenshot options.
     *
     * @param {FormData} data - The FormData object to be customized.
     * @param {ScreenshotOptions} options - The screenshot options to apply to the FormData.
     * @returns {Promise<void>} A Promise that resolves once the customization is complete.
     */
    static customize(data, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (options.header) {
                const { header } = options;
                yield common_1.GotenbergUtils.addFile(data, header, 'header.html');
            }
            if (options.footer) {
                const { footer } = options;
                yield common_1.GotenbergUtils.addFile(data, footer, 'footer.html');
            }
            if (options.emulatedMediaType) {
                data.append('emulatedMediaType', options.emulatedMediaType);
            }
            if (options.properties) {
                ScreenshotUtils.addImageProperties(data, options.properties);
            }
            if (options.waitDelay) {
                data.append('waitDelay', options.waitDelay);
            }
            if (options.waitForExpression) {
                data.append('waitForExpression', options.waitForExpression);
            }
            if (options.extraHttpHeaders) {
                data.append('extraHttpHeaders', JSON.stringify(options.extraHttpHeaders));
            }
            if (options.failOnHttpStatusCodes) {
                data.append('failOnHttpStatusCodes', JSON.stringify(options.failOnHttpStatusCodes));
            }
            if (options.failOnConsoleExceptions) {
                data.append('failOnConsoleExceptions', String(options.failOnConsoleExceptions));
            }
            if (options.skipNetworkIdleEvent) {
                data.append('skipNetworkIdleEvent', String(options.skipNetworkIdleEvent));
            }
            if (options.optimizeForSpeed) {
                data.append('optimizeForSpeed', String(options.optimizeForSpeed));
            }
        });
    }
}
exports.ScreenshotUtils = ScreenshotUtils;
//# sourceMappingURL=screenshot.utils.js.map