"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GotenbergUtils = void 0;
const tslib_1 = require("tslib");
const fs_1 = require("fs");
const node_fetch_1 = tslib_1.__importDefault(require("node-fetch"));
/**
 * Utility class for common tasks related to the Gotenberg service.
 */
class GotenbergUtils {
    /**
     * Asserts that a condition is true; otherwise, throws an error with the specified message.
     *
     * @param {boolean} condition - The condition to assert.
     * @param {string} message - The error message to throw if the condition is false.
     * @throws {Error} Throws an error with the specified message if the condition is false.
     */
    static assert(condition, message) {
        if (!condition) {
            throw new Error(message);
        }
    }
    /**
     * Performs a POST request to the specified Gotenberg endpoint with the provided FormData.
     *
     * @param {string} endpoint - The Gotenberg endpoint URL.
     * @param {FormData} data - The FormData object to be sent in the POST request.
     * @returns {Promise<Buffer>} A Promise that resolves to the response body as a Buffer.
     * @throws {Error} Throws an error if the HTTP response status is not OK.
     */
    static fetch(endpoint, data) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const response = yield (0, node_fetch_1.default)(endpoint, {
                method: 'post',
                body: data,
                headers: Object.assign({}, data.getHeaders())
            });
            if (!response.ok) {
                throw new Error(`${response.status} ${response.statusText}`);
            }
            return response.buffer();
        });
    }
    /**
     * Adds a file to the FormData object.
     *
     * @param {FormData} data - The FormData object to which the file will be added.
     * @param {PathLikeOrReadStream} file - The file to be added (either a PathLike or a ReadStream).
     * @param {string} name - The name to be used for the file in the FormData.
     * @returns {Promise<void>} A Promise that resolves once the file has been added.
     */
    static addFile(data, file, name) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (Buffer.isBuffer(file)) {
                data.append('files', file, name);
            }
            else if (file instanceof fs_1.ReadStream) {
                data.append('files', file, name);
            }
            else {
                yield fs_1.promises.access(file, fs_1.constants.R_OK);
                data.append('files', (0, fs_1.createReadStream)(file), name);
            }
        });
    }
}
exports.GotenbergUtils = GotenbergUtils;
//# sourceMappingURL=gotenberg.utils.js.map