"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LibreOfficeUtils = void 0;
const tslib_1 = require("tslib");
const fs_1 = require("fs");
const path_1 = tslib_1.__importDefault(require("path"));
const file_type_1 = require("file-type");
const common_1 = require("../../common");
const constants_1 = require("./constants");
/**
 * Utility class for handling common tasks related to LibreOffice conversions.
 */
class LibreOfficeUtils {
    /**
     * Adds files to the FormData object for LibreOffice conversion.
     *
     * @param {PathLikeOrReadStream[]} files - An array of files to be added to the FormData.
     * @param {FormData} data - The FormData object to which files will be added.
     * @throws {Error} Throws an error if the file extension is not supported.
     */
    static addFiles(files, data) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            for (const [key, value] of files.entries()) {
                let file = value;
                let fileInfo;
                if (Buffer.isBuffer(value)) {
                    fileInfo = yield (0, file_type_1.fromBuffer)(value);
                }
                else if (value instanceof fs_1.ReadStream) {
                    fileInfo = yield (0, file_type_1.fromStream)(value);
                }
                else {
                    yield fs_1.promises.access(value, fs_1.constants.R_OK);
                    const filename = path_1.default.basename(value.toString());
                    fileInfo = { ext: path_1.default.extname(filename).slice(1) };
                    file = (0, fs_1.createReadStream)(value);
                }
                if (!fileInfo) {
                    throw new Error('File type could not be determined');
                }
                const extension = fileInfo.ext;
                if (constants_1.LIBRE_OFFICE_EXTENSIONS.includes(extension)) {
                    data.append('files', file, `${key}.${extension}`);
                }
                else {
                    throw new Error(`${extension} is not supported`);
                }
            }
        });
    }
    /**
     * Adds page properties to the FormData object based on the provided PageProperties.
     *
     * @param {FormData} data - The FormData object to which page properties will be added.
     * @param {PageProperties} pageProperties - The page properties to be added to the FormData.
     */
    static addPageProperties(data, pageProperties) {
        if (pageProperties.landscape) {
            data.append('landscape', String(pageProperties.landscape));
        }
        if (pageProperties.nativePageRanges) {
            common_1.GotenbergUtils.assert(pageProperties.nativePageRanges.from > 0 &&
                pageProperties.nativePageRanges.to > 0 &&
                pageProperties.nativePageRanges.to >=
                    pageProperties.nativePageRanges.from, 'page ranges syntax error');
            data.append('nativePageRanges', `${pageProperties.nativePageRanges.from}-${pageProperties.nativePageRanges.to}`);
        }
    }
}
exports.LibreOfficeUtils = LibreOfficeUtils;
//# sourceMappingURL=libre-office.utils.js.map